"""Environment variable(s) for Connect-N API"""

MINIMUM_REQUEST_DURATION = 5

API_DB_NAME = "/api.db"

RESPONSE_TEMPLATE = {
    "@author": "Kartikei Mittal",
    "project_link": "https://github.com/Kartikei-12/Connect-N",
    "description": "",
}
