"""Default and environment variables for this project."""

N = 4
ROWS = 6
COLUMNS = 7

OFFSET = -2 * (10 ** 3)
UNIT_SCORE = 10 ** 3

LARGE_VALUE = 10 ** 12
SMALL_VALUE = -10 ** 12

NEURAL_NET_param = {"name": "Neural Network parameters"}
